"""copier_template_tester."""

from ._runtime_type_check_setup import configure_runtime_type_checking_mode

__version__ = '2.1.4'
__pkg_name__ = 'copier_template_tester'

configure_runtime_type_checking_mode()


# == Above code must always be first ==
